// Requis
var gulp = require('gulp');

// Include plugins
var jshint = require('gulp-jshint');
var concat = require('gulp-concat');
var uglify = require('gulp-uglify');
var rename = require('gulp-rename');
var autoprefixer = require('gulp-autoprefixer');
var pleeease = require('gulp-pleeease');
var concatCss = require('gulp-concat-css');
var cleanCSS = require('gulp-clean-css');
var imagemin = require('gulp-imagemin');
var cache = require('gulp-cache');
var plumber = require('gulp-plumber');
var notify = require('gulp-notify');
var less = require('gulp-less');

var plumberErrorHandler = {
    errorHandler: notify.onError({
        title: 'Gulp',
        message: 'Error: <%= error.message %>'
    })
};

var paths = {
    scripts: [
        'js/main.js',
    ],
    stylesWatchDir: [
        'styles/*.less'
    ],
    styles: [
        'styles/styles.less'
    ],
    images:'images',
    dest:'dist'
};

// Lint Task
gulp.task('lint', function() {
  return gulp.src(paths.scripts)
        .pipe(plumber(plumberErrorHandler))
        .pipe(jshint())
        .pipe(jshint.reporter('default'));
});

// Concatenate & Minify JS
gulp.task('scripts', function() {
  return gulp.src(paths.scripts)
        .pipe(plumber(plumberErrorHandler))
        .pipe(concat('all.js'))
        .pipe(gulp.dest(paths.dest+'/js'))
        .pipe(rename('all.min.js'))
        .pipe(uglify({ mangle: false }))
        .pipe(gulp.dest(paths.dest+'/js'));
});

// Concatenate & Minify CSS
gulp.task('css', function () {
  return gulp.src(paths.styles)
        .pipe(plumber(plumberErrorHandler))
        .pipe(less())
        .pipe(pleeease({browsers: ['last 4 versions'], minifier: false}))
        .pipe(concatCss('style.css'))
        .pipe(gulp.dest(paths.dest+'-css/'))
        .pipe(rename('style.min.css'))
        .pipe(cleanCSS({}))
        .pipe(gulp.dest(paths.dest+'-css/'));
});

gulp.task('images', function() {
    return gulp.src(paths.images+'/**/*')
        .pipe(plumber(plumberErrorHandler))
        .pipe(cache(imagemin({ optimizationLevel: 5, progressive: true, interlaced: true })))
        .pipe(gulp.dest(paths.dest+'-images/'));
});

// Watch Files For Changes
gulp.task('watch', function() {
    gulp.watch(paths.scripts, ['scripts']);
    gulp.watch(paths.stylesWatchDir, ['css']);
    gulp.watch(paths.images+'/**/*', ['images']);
});

// Clear cache
gulp.task('clear', function(){
    cache.clearAll();
});

// Default Task
gulp.task('default', ['scripts', 'css', 'images', 'watch']);

gulp.task('delivery', ['scripts', 'css', 'images']);
